@@ -15,6 +15,7 @@
  */
 #include<cstdio>
 #include<cstring>
+#include<cstdarg>
 #include<fstream>
 #include<functional>
 #include<iomanip>
@@ -149,6 +150,13 @@ TSPIException::code_to_extra(int code)
       "  Make sure trousers is started (/etc/init.d/trousers start) correctly, and\n"
       "  and check any logs for why it might not be coming up correctly.\n"
       "  It could fail to start because it's not finding a device /dev/tpm*.";
+  case TSS_E_PS_KEY_NOTFOUND:
+    return "Likely problem:\n"
+      "  The TPM chip is not active. Use tpm_getpubek to see if its error message\n"
+      "  confirms this.\n"
+      "Possible solution:\n"
+      "  Power off the machine, power it back on, go into BIOS, and make sure the\n"
+      "  TPM chip / security chip is \"Active\".";
   }
   return "";
 }
@@ -197,6 +205,24 @@ xctime()
   return buf;
 }
 
+std::string
+xsprintf(const char* fmt, ...) {
+  va_list args;
+  va_start(args, fmt);
+
+  va_list va2;
+  va_copy(va2, args);
+
+  size_t s = vsnprintf(NULL, 0, fmt, args) + 1;
+  va_end(args);
+
+  std::vector<char> buf(s);
+  vsnprintf(&buf[0], s, fmt, va2);
+  va_end(va2);
+
+  return std::string(buf.begin(), buf.end());
+}
+
 bool
 log_stderr()
 {
